<?php
/*
 * @copyright 2023 Passwords Handbook App
 *
 * @author Marius David Wieschollek
 * @license AGPL-3.0
 *
 * This file is part of the Passwords App
 * created by Marius David Wieschollek.
 */

namespace OCA\PasswordsHandbook\Controller\Admin;

use JetBrains\PhpStorm\ArrayShape;
use OCA\Passwords\Exception\ApiException;
use OCA\PasswordsHandbook\Service\HandbookUrlService;
use OCA\PasswordsHandbook\Service\SetupCheckService;
use OCP\AppFramework\ApiController;
use OCP\AppFramework\Http\JSONResponse;
use OCP\IRequest;

class SettingsController extends ApiController {

    /**
     * @param string   $appName
     * @param IRequest $request
     */
    public function __construct(
        string                       $appName,
        IRequest                     $request,
        protected HandbookUrlService $handbookUrlService,
        protected SetupCheckService  $setupCheckService
    ) {
        parent::__construct(
            $appName,
            $request,
            'PUT, GET, DELETE'
        );
    }

    /**
     * @return JSONResponse
     */
    public function index(): JSONResponse {
        $settings = [
            $this->makeSetting('enabled', $this->handbookUrlService->isUrlSet())
        ];

        return new JSONResponse($settings);
    }

    /**
     * @param $id
     *
     * @return JSONResponse
     * @throws ApiException
     */
    public function show($id): JSONResponse {
        $data = [];
        if($id === 'enabled') {
            $data = $this->makeSetting('enabled', $this->handbookUrlService->isUrlSet());
        }

        return new JSONResponse($data);
    }

    /**
     * @param $id
     * @param $value
     *
     * @return JSONResponse
     * @throws ApiException
     */
    public function update($id, $value): JSONResponse {
        if($id === 'enabled') {
            if($value) {
                $this->handbookUrlService->setUrl();
            } else {
                $this->handbookUrlService->resetUrl();
            }
        }

        $data          = $this->makeSetting('enabled', $this->handbookUrlService->isUrlSet());
        $data['url']   = $this->handbookUrlService->getCurrentUrl();
        $data['error'] = $this->setupCheckService->check();

        return new JSONResponse($data);
    }

    /**
     * @param $id
     *
     * @return JSONResponse
     * @throws ApiException
     */
    public function destroy($id): JSONResponse {
        if($id === 'enabled') {
            $this->handbookUrlService->resetUrl();
        }

        return $this->show($id);
    }

    /**
     * @param $id
     * @param $value
     *
     * @return array
     */
    #[ArrayShape(['name' => "", 'value' => ""])] protected function makeSetting($id, $value): array {
        return [
            'name'  => $id,
            'value' => $value
        ];
    }
}